/**
 * 登录
 */
import { chatLogin } from '../../chat/client/app/net/login';
import { earnLogin } from '../../earn/client/app/net/login';
import { App } from '../../pi/browser/app';
import { DeviceIdProvider } from '../../pi/browser/device';
import { request } from '../../pi/net/ui/con_mgr';
import { getStoreData } from '../api/walletApi';
import { getStore, setStore } from '../store/memstore';
import { popNewMessage } from '../utils/pureUtils';
import { getAllGame, getGameInfo, getHotGame, getRecommendationsList, getUserRecentGame } from './pull';

/**
 * 通用的异步通信
 */
export const requestAsync = (msg: any): Promise<any> => {
    return new Promise((resolve, reject) => {
        request(msg, (resp: any) => {
            if (resp.type) {
                console.log(`错误信息为${resp.type}`);
                reject(resp);
            } else if (resp.result !== 1) {
                reject(resp);
            } else {
                resolve(resp);
            }
        });
    });
};

// 钱包登录
export const walletLogin = (cb?: Function) => {
    console.log('walletLoginIng');
    (<any>window).pi_sdk.platform.authorize({ appId: '101' }, async (err, result) => {
        console.log('authorize', err, JSON.stringify(result));
        cb && cb();

        if (!err) {
            setStore('flags/authorized', true);
            console.log('钱包注册成功', result);
            // 获取全部游戏
            getAllGameList();

            // 获取当前版本号
            App.getVersion((err, version) => {
                if (!err) {
                    setStore('flags/appVersion', version);
                }
            });
        } else {
            if (err === -3) { // vm正在登录中
                popNewMessage('授权中，请稍候');
            }
            setStore('flags/authorized', false);
        }
    });
};

/**
 * 判断VM中是否已经有账号
 * 有账号则执行授权，无账号则等到触发事件时执行
 */
export const checkAccount = async (cb: Function) => {
    const conUid = await getStoreData('user/token', '');
    if (conUid) {  // 已有账号执行授权
        walletLogin(cb);
    }
};

/**
 * 判断是否已经授权成功
 */
export const checkAuthorize = () => {
    const flags = getStore('flags', {});
    if (!flags.authorized) {  // 未授权则执行授权    
        walletLogin();
        chatLogin();
        earnLogin();

        return false;
    }

    return true;
};

/**
 * 获取游戏列表
 */
const getAllGameList = async () => {
    try {
        const gameList = getStore('game', {});

        // 全部游戏
        const allList: string[] = await getAllGame();
        if (!allList) return; // 全部游戏没有数据，那其他也不会有
        const appIds = JSON.stringify(allList);
        const allGameInfos = await getGameInfo(appIds);
        console.log('全部游戏', allGameInfos);
        gameList.allGame = allGameInfos;

        // 最近在玩
        const user = await getStoreData('user', {});
        const recentList: string[] = await getUserRecentGame(user.accId, 10);
        const recentGameData = allGameInfos.filter(v => {
            return recentList.indexOf(v.appid) > -1;
        });
        console.log('最近在玩游戏', recentGameData);
        gameList.oftenGame = recentGameData;

        // 热门游戏
        const hotList: string[] = await getHotGame();
        const hotGameData = allGameInfos.filter(v => {
            return hotList.indexOf(v.appid) > -1;
        });
        console.log('热门游戏', hotGameData);
        gameList.hotGame = hotGameData;

        // 推荐游戏
        const recomList: string[] = await getRecommendationsList();
        const recomGameData = allGameInfos.filter(v => {
            return recomList.indexOf(v.appid) > -1;
        });
        console.log('推荐游戏', recomGameData);
        gameList.recommendGame = recomGameData;

        // 今日推荐游戏（本包推荐）
        const divice = new DeviceIdProvider();
        divice.getGameName((name: string) => {
            console.log(`获取包名 = ${name}`);
            const todayGame = allGameInfos.find(item => item.webviewName === name);
            console.log('今日推荐游戏', todayGame);
            setStore('game/recommendedToday', todayGame ? [todayGame] : allGameInfos.slice(0, 1));
        });

        setStore('game', gameList);

    } catch (err) {
        console.warn('获取游戏列表失败', err);
        popNewMessage('获取游戏列表失败');
    }

};