/**
 * @file store
 * @author donghr
 */
// ============================================ 导入
import { appLanguageList } from '../../pi/browser/localLanguage';
import { HandlerMap } from '../../pi/util/event';
import { defaultSetting, topHeight } from '../public/config';
import { GameList } from '../public/constant';
import { Store } from '../public/interface';
import { getSystemLanguage } from '../utils/native';
import { getLocalStorage, setLocalStorage } from './filestore';

// ============================================ 导出

/**
 * 数据深拷贝
 */
export const deepCopy = (v: any): any => {

    return _.cloneDeep(v);  // lodash文件中的方法
    
    // if (!v || typeof v !== 'object') {
    //     return v;
    
    // } else if (v instanceof Map) {
    //     return new Map(v);
    
    // } else {
    //     return JSON.parse(JSON.stringify(v));
        
    // }
};

/**
 * 根据路径获取数据
 * 深拷贝 只支持一层map
 */
export const getStore = (path: string, defaultValue = undefined) => {
    let ret = store;
    for (const key of path.split('/')) {
        if (key in ret) {
            ret = ret[key];

        } else if (ret instanceof Map) {
            ret = ret.get(key);
            
        } else {
            // 路径中有和store内部不同的键，肯定是bug
            console.warn(`getStore Failed, path = ${path}`);
            ret = defaultValue;

            break;
        }
    }

    return deepCopy(ret) || defaultValue;
};

/**
 * 更新store并通知
 */
export const setStore = (path: string, data: any, notified = true) => {
    const keyArr = path.split('/');

    const notifyPath = [];
    for (let i = 0; i < keyArr.length; i++) {
        // tslint:disable-next-line:prefer-template
        const path = i === 0 ? keyArr[i] : notifyPath[i - 1] + '/' + keyArr[i];
        notifyPath.push(path);
    }

    // 原有的最后一个键
    const lastKey = keyArr.pop();

    let parent = store;
    for (const key of keyArr) {
        if (key in parent) {
            parent = parent[key];
        } else if (parent instanceof Map) {
            parent = parent.get(key);
        } else {
            // 路径中有和store内部不同的键，肯定是bug
            console.warn(`setStore Failed, path = ${path}`);
        }
    }
    let oldData;   // 原始数据
    if (parent instanceof Map) {
        oldData = deepCopy(parent.get(lastKey));
        parent.set(lastKey, deepCopy(data));
    } else {
        oldData = deepCopy(parent[lastKey]);
        parent[lastKey] = deepCopy(data);
    }

    /**
     * 只能支持普通json和最后一层为map的情况，不支持map嵌套
     * 非map，比较数据有变化才通知，map不比较直接通知
     */
    if (notified && (oldData instanceof Map || JSON.stringify(oldData) !== JSON.stringify(data))) {
        for (let i = notifyPath.length - 1; i >= 0; i--) {
            handlerMap.notify(notifyPath[i], getStore(notifyPath[i]));
        }
        
    }

};

/**
 * 注册消息处理器
 */
export const register = (keyName: string, cb: Function, directReturn:boolean = true): void => {
    handlerMap.add(keyName, <any>cb);
    if (directReturn) {
        cb && cb(getStore(keyName));
    }
};

/**
 * 取消注册消息处理器
 */
export const unregister = (keyName: string, cb: Function): void => {
    handlerMap.remove(keyName, <any>cb);
};

// ===================================================本地

/**
 * 初始化store
 */
export const initStore = () => {
    registerFileStore();    // 注册监听 

    // 初始化setting
    const setting = getLocalStorage('setting');
    if (setting) {
        setStore('setting', setting);
    }
    // 获取系统语言设置 TODO
    // getSystemLanguage(localLan => {
    //     const language = appLanguageList[Number(localLan)];
    //     if (language) {
    //         setStore('setting/language', language);
    //     }
    // });
    // 游戏数据初始化   
    const game = getLocalStorage('game');
    if (game) {
        setStore('game', game);
    }
};

// ===================================================本地

/**
 * 注册文件数据库监听
 */
const registerFileStore = () => {
    register('setting', (r) => {
        setLocalStorage('setting', r);
    },false);
    
    register('game', (r:any) => {
        setLocalStorage('game',r);
    },false);
};
// ======================================================== 本地

// ============================================ 立即执行

/**
 * 消息处理列表
 */
const handlerMap: HandlerMap = new HandlerMap();

// 全局内存数据库
const store: Store = {
    activity: {
        luckyMoney: {
            sends: null,          // 发送红包记录
            exchange: null,       // 兑换红包记录
            invite: null          // 邀请码记录
        },
        dividend: {
            total: null,         // 分红汇总信息
            history: null       // 分红历史记录
        }
    },
    setting: {
        language: defaultSetting.DEFAULT_LANGUAGE,             // 语言
        topHeight,              // 设备头部应空出来的高度
        bottomHeight: 0            // 设备底部应空出来的高度
    },
    game:{
        allGame:GameList,// 全部游戏
        hotGame:[],// 热门游戏
        oftenGame:[],// 最近在玩的游戏
        recommendGame:[],// 推荐游戏
        recommendedToday:[]// 今日推荐
    },
    flags: {}
};

// =======================localStorage interface===============================

initStore();