/**
 * 主动向钱包通讯
 */
// ===================================================== 导入
import { isNumber } from '../../pi/util/util';
import { ERC20Tokens } from '../config';
import { BtcApi } from '../core/btc/api';
import { BTCWallet } from '../core/btc/wallet';
import { Api as EthApi } from '../core/eth/api';
import { EthWallet, initWeb3, web3 } from '../core/eth/wallet';
import { GlobalWallet } from '../core/globalWallet';
import { dataCenter } from '../logic/dataCenter';
import { MinerFeeLevel, TxHistory, TxStatus, TxType } from '../store/interface';
import { erc20GasLimitRate } from '../utils/constants';
// tslint:disable-next-line:max-line-length
import { deletLocalTx, fetchBtcMinerFee, fetchGasPrice, getConfirmBlockNumber, getCurrentEthAddr, getEthNonce, getStaticLanguage, popNewLoading, popNewMessage, setEthNonce, updateLocalTx } from '../utils/tools';
import { btc2Sat, eth2Wei, ethTokenMultiplyDecimals, wei2Eth } from '../utils/unitTools';
import { fetchMinerFeeList, getWltAddrIndex, VerifyIdentidy } from '../utils/walletTools';
// tslint:disable-next-line:max-line-length
import { btcRechargeToServer, btcWithdrawFromServer, getBankAddr, getBtcBankAddr, getRechargeLogs, getWithdrawLogs, rechargeToServer, withdrawFromServer } from './pull';
// ===================================================== 导出
export interface TxPayload {
    fromAddr: string;        // 转出地址
    toAddr: string;          // 转入地址
    pay: number;             // 转账金额
    currencyName: string;    // 转账货币
    fee: number;             // 矿工费
    minerFeeLevel: MinerFeeLevel;   // 矿工费等级
}

export interface TxPayload3 {
    fromAddr: string;        // 转出地址
    toAddr: string;          // 转入地址
    pay: string | number;             // 转账金额
    currencyName: string;    // 转账货币
    data: string;
}

/**
 * 供其他的webview调用
 */
export const rpcProviderSendAsync = (payload, callback) => {
    initWeb3();
    if (payload.method === 'eth_accounts') {
        let addr = getCurrentEthAddr();
        addr = addr ? [addr] : [];
        callback(null, { jsonrpc: '2.0', result: addr, id: payload.id });
    } else if (payload.method === 'eth_sendTransaction') {
        // alert(`payload is ${JSON.stringify(payload)}`);
        const ethPayload = {
            fromAddr: payload.params[0].from,
            toAddr: payload.params[0].to,
            pay: payload.params[0].value,
            currencyName: 'ETH',
            data: payload.params[0].data
        };
        try {

            const promise = transfer3(payload.passwd, ethPayload);

            promise.then(([err, hash]) => {
                console.log(`wallet rpcProviderSendAsync err is ${err}, hash is ${hash}`);
                if (err) {
                    callback(err);
                } else {
                    callback(null, { jsonrpc: '2.0', result: hash, id: payload.id });
                }
            }).catch((err) => {
                console.log(`wallet rpcProviderSendAsync err is catch`);
                callback(err);
            });
        } catch (e) {
            console.log(`transfer3 catch throw`);
            callback(e);
        }

    } else {
        if (web3 && web3.currentProvider && web3.currentProvider.sendAsync) {
            web3.currentProvider.sendAsync(payload, callback);
        }
    }

};

/**
 * 普通转账
 */
const transfer3 = async (psw: string, txPayload: TxPayload3) => {
    try {
        if (psw.length <= 0) return ['have no password'];
        const minerFeeList = fetchMinerFeeList(txPayload.currencyName);
        const txRecord: TxHistory = {
            hash: '',
            addr: txPayload.fromAddr,
            txType: TxType.Transfer,
            fromAddr: txPayload.fromAddr,
            toAddr: txPayload.toAddr,
            pay: wei2Eth(txPayload.pay),
            time: 0,
            status: TxStatus.Pending,
            confirmedBlockNumber: 0,
            needConfirmedBlockNumber: 0,
            info: '',
            currencyName: txPayload.currencyName,
            fee: minerFeeList[MinerFeeLevel.Standard].minerFee,
            nonce: undefined,
            minerFeeLevel: MinerFeeLevel.Standard
        };
        const tx = await transferCore(psw, txRecord);

        return [undefined, tx.hash];
    } catch (error) {
        return [error, undefined];
    }
};

/**
 * 转账核心模块
 * @param psw 密码
 * @param txRecord 交易记录
 */
const transferCore = async (psw: string, txRecord: TxHistory) => {
    const fromAddr = txRecord.fromAddr;
    const currencyName = txRecord.currencyName;
    let ret: any;
    const addrIndex = getWltAddrIndex(fromAddr, currencyName);
    if (addrIndex < 0) throw new Error('Invalid address');
    const wlt = await GlobalWallet.createWlt(currencyName, psw, addrIndex);
    if (currencyName === 'ETH') {
        ret = await doEthTransfer(<EthWallet>wlt, txRecord);
    } else if (currencyName === 'BTC') {
        const res = await doBtcTransfer(<BTCWallet>wlt, txRecord);
        if (!res) throw new Error('btc send failed');
        ret = { hash: res.txid, nonce: -1 };
    } else if (ERC20Tokens[currencyName]) {
        // tslint:disable-next-line:max-line-length
        const transferCode = EthWallet.tokenOperations('transfer', currencyName, txRecord.toAddr, ethTokenMultiplyDecimals(txRecord.pay, currencyName));
        ret = await doEthTransfer(<EthWallet>wlt, txRecord, transferCode);
    }
    const tx = {
        ...txRecord,
        hash: ret.hash,
        nonce: ret.nonce,
        time: new Date().getTime()
    };

    return tx;
};

/**
 * 处理ETH转账
 */
const doEthTransfer = async (wlt: EthWallet, txRecord: TxHistory, transferCode?: string) => {
    const oldNonce = txRecord.nonce;
    const api = new EthApi();
    let newNonce;
    let gasLimit;
    let chainNonce;
    // tslint:disable-next-line:max-line-length
    const gasLimitPromise = transferCode ? estimateGasERC20(txRecord.currencyName, txRecord.toAddr, txRecord.fromAddr, '0x0') : estimateGasETH(txRecord.toAddr, txRecord.info);
    if (!isNumber(oldNonce)) {
        const localNonce = getEthNonce(txRecord.fromAddr);
        [chainNonce, gasLimit] = await Promise.all([api.getTransactionCount(txRecord.fromAddr), gasLimitPromise]);
        newNonce = localNonce >= chainNonce ? localNonce : chainNonce;
    } else {  // 重发
        newNonce = oldNonce;
        gasLimit = await gasLimitPromise;
    }
    if (transferCode) gasLimit = Math.floor(gasLimit * erc20GasLimitRate);    // 代币交易gasLimit估值不准 暂时方案

    const hash = await api.sendRawTransaction(wlt.signRawTransaction({
        to: transferCode ? ERC20Tokens[txRecord.currencyName].contractAddr : txRecord.toAddr,
        nonce: newNonce,
        gasPrice: fetchGasPrice(txRecord.minerFeeLevel || MinerFeeLevel.Standard),
        gasLimit,
        value: transferCode ? eth2Wei(0) : eth2Wei(txRecord.pay),
        data: transferCode || txRecord.info
    }));
    if (!isNumber(oldNonce) && hash) {
        setEthNonce(newNonce + 1, txRecord.fromAddr);
    }

    return {
        hash,
        nonce: newNonce
    };

};

/**
 * 处理BTC转账
 */
const doBtcTransfer = async (wlt: BTCWallet, txRecord: TxHistory) => {
    const hash = txRecord.hash;
    const minerFee = fetchBtcMinerFee(txRecord.minerFeeLevel);
    wlt.unlock();
    await wlt.init();
    let retArr;
    if (!hash) {
        // tslint:disable-next-line:max-line-length
        retArr = await wlt.buildRawTransactionFromSingleAddress(txRecord.fromAddr, { toAddr: txRecord.toAddr, amount: txRecord.pay, chgAddr: txRecord.fromAddr }, minerFee);
    } else {
        retArr = await wlt.resendTx(hash, minerFee);
    }
    wlt.lock();

    return BtcApi.sendRawTransaction(retArr.rawTx);
};

/**
 * 普通转账
 */
export const transfer = async (psw: string, txPayload: TxPayload) => {
    const fromAddr = txPayload.fromAddr;
    const currencyName = txPayload.currencyName;
    const needConfirmedBlockNumber = getConfirmBlockNumber(currencyName, txPayload.pay);
    const txRecord: TxHistory = {
        hash: '',
        addr: fromAddr,
        txType: TxType.Transfer,
        fromAddr,
        toAddr: txPayload.toAddr,
        pay: txPayload.pay,
        time: 0,
        status: TxStatus.Pending,
        confirmedBlockNumber: 0,
        needConfirmedBlockNumber,
        info: '',
        currencyName,
        fee: txPayload.fee,
        nonce: undefined,
        minerFeeLevel: txPayload.minerFeeLevel
    };
    const tx = await transferCore(psw, txRecord);
    updateLocalTx(tx);
    dataCenter.updateAddrInfo(tx.addr, tx.currencyName);

    return tx;
};

/**
 * 普通转账重发
 */
export const resendNormalTransfer = async (psw: string, txRecord: TxHistory) => {
    const oldHash = txRecord.hash;
    const tx = await transferCore(psw, txRecord);
    tx.oldHash = oldHash;
    updateLocalTx(tx);
    dataCenter.updateAddrInfo(tx.addr, tx.currencyName);

    return tx;
};

// =====================================================ETH
/**
 * 预估ETH的gas limit
 */
export const estimateGasETH = async (toAddr: string, data?: any) => {
    const api = new EthApi();

    return api.estimateGas({ to: toAddr, data });
};

/**
 * ETH交易签名
 */
const signRawTransactionETH = async (psw: string, txRecord: TxHistory, gasLimit: number) => {
    const addrIndex = getWltAddrIndex(txRecord.fromAddr, 'ETH');
    if (addrIndex < 0) throw new Error('Invalid address');
    const wlt = await GlobalWallet.createWlt('ETH', psw, addrIndex);
    const api = new EthApi();
    let oldNonce = txRecord.nonce;
    let newNonce;
    if (!isNumber(oldNonce)) {
        const localNonce = getEthNonce(txRecord.fromAddr);
        const chainNonce = await api.getTransactionCount(txRecord.fromAddr);
        newNonce = localNonce >= chainNonce ? localNonce : chainNonce;
    } else {  // 重发
        newNonce = oldNonce;
    }
    const txObj = {
        to: txRecord.toAddr,
        nonce: newNonce,
        gasPrice: fetchGasPrice(txRecord.minerFeeLevel),
        gasLimit,
        value: eth2Wei(txRecord.pay),
        data: txRecord.info
    };
    console.log('txObj--------------', txObj);

    return (<EthWallet>wlt).signRawTransactionHash(txObj);

};

/**
 * 发送ETH交易
 * @param signedTx 签名交易
 */
const sendRawTransactionETH = (signedTx: string) => {
    const api = new EthApi();
    return api.sendRawTransaction(signedTx);
};

// ==============================================ERC20
// 预估ETH ERC20Token的gas limit
export const estimateGasERC20 = (currencyName: string, toAddr: string, fromAddr: string, amount: number | string) => {
    const api = new EthApi();

    const transferCode = EthWallet.tokenOperations('transfer', currencyName, toAddr, ethTokenMultiplyDecimals(amount, currencyName));

    return api.estimateGas({ to: ERC20Tokens[currencyName].contractAddr, from: fromAddr, value: '0x0', data: transferCode });
};

// ==================================================BTC
// 预估BTC矿工费
export const estimateMinerFeeBTC = async (nbBlocks: number = 12) => {
    return BtcApi.estimateFee(nbBlocks);
};

/**
 * BTC交易签名
 */
export const signRawTransactionBTC = async (psw: string, fromAddr: string, toAddr: string,
    pay: number, minerFee: number) => {
    const addrIndex = getWltAddrIndex(fromAddr, 'BTC');
    if (addrIndex < 0) throw new Error('Invalid address');
    const wlt = await GlobalWallet.createWlt('BTC', psw, addrIndex);
    const output = {
        toAddr,
        amount: pay,
        chgAddr: fromAddr
    };
    console.log('output----------------', output);
    (<BTCWallet>wlt).unlock();
    await (<BTCWallet>wlt).init();

    const retArr = await (<BTCWallet>wlt).buildRawTransactionFromSingleAddress(fromAddr, output, minerFee);
    (<BTCWallet>wlt).lock();

    return retArr;

};

/**
 * BTC重发交易签名
 */
export const resendSignRawTransactionBTC = async (hash: string, psw: string, fromAddr: string, minerFee: number) => {
    const addrIndex = getWltAddrIndex(fromAddr, 'BTC');
    if (addrIndex < 0) throw new Error('Invalid address');
    const wlt = await GlobalWallet.createWlt('BTC', psw, addrIndex);
    (<BTCWallet>wlt).unlock();
    await (<BTCWallet>wlt).init();
    const retArr = await (<BTCWallet>wlt).resendTx(hash, minerFee);
    (<BTCWallet>wlt).lock();
    return retArr;

};

/**
 * 发送BTC交易
 * @param signedTx 签名交易
 */
export const sendRawTransactionBTC = (rawHexString: string) => {
    return BtcApi.sendRawTransaction(rawHexString);
};

// ===================================================== 本地

// ================================重发

/**
 * 充值重发
 */
export const resendRecharge = async (psw: string, txRecord: TxHistory) => {
    const oldTx = { ...txRecord };
    let tx;
    if (txRecord.currencyName === 'BTC') {
        tx = await btcRecharge(psw, txRecord);
    } else {
        tx = await ethRecharge(psw, txRecord);
    }
    tx.oldHash = oldTx.hash;
    oldTx.reHash = tx.hash;
    updateLocalTx(tx);
    updateLocalTx(oldTx);
    dataCenter.updateAddrInfo(tx.addr, tx.currencyName);
    getRechargeLogs(tx.currencyName);
    return tx.hash;
};

// ================================重发

export const recharge = async (psw: string, txRecord: TxHistory) => {
    let tx;
    if (txRecord.currencyName === 'BTC') {
        tx = await btcRecharge(psw, txRecord);
    } else {
        tx = await ethRecharge(psw, txRecord);
    }
    updateLocalTx(tx);
    dataCenter.updateAddrInfo(tx.addr, tx.currencyName);
    getRechargeLogs(tx.currencyName);
    return tx.hash;
};
/**
 * eth充值
 */
export const ethRecharge = async (psw: string, txRecord: TxHistory) => {
    const oldNonce = txRecord.nonce;
    const toAddr = await getBankAddr();
    txRecord.toAddr = toAddr;
    const gasLimit = await estimateGasETH(txRecord.toAddr, txRecord.info);
    const obj = await signRawTransactionETH(psw, txRecord, gasLimit);
    const signedTX = obj.signedTx;
    const hash = `0x${obj.hash}`;
    const nonce = Number(obj.nonce);
    await rechargeToServer(txRecord.fromAddr, toAddr, hash, nonce, fetchGasPrice(txRecord.minerFeeLevel), eth2Wei(txRecord.pay));
    await sendRawTransactionETH(signedTX);
    if (!isNumber(oldNonce)) {
        setEthNonce(nonce + 1, txRecord.fromAddr);
    }
    txRecord.nonce = nonce;
    txRecord.hash = hash;
    txRecord.time = new Date().getTime();
    txRecord.fee = wei2Eth(gasLimit * fetchGasPrice(txRecord.minerFeeLevel));
    return txRecord;
};

/**
 * btc充值
 */
export const btcRecharge = async (psw: string, txRecord: TxHistory) => {
    const oldHash = txRecord.hash;
    const toAddr = await getBtcBankAddr();
    txRecord.fee = fetchBtcMinerFee(txRecord.minerFeeLevel);
    let signedTX;
    let newHash;
    if (oldHash) { // 重发
        const ret = await resendSignRawTransactionBTC(txRecord.hash, psw, txRecord.fromAddr, txRecord.fee);
        newHash = ret.newTxid;
        signedTX = ret.rawTx;
    } else {
        const obj = await signRawTransactionBTC(psw, txRecord.fromAddr, toAddr, txRecord.pay, txRecord.fee);
        signedTX = obj.rawTx;
        newHash = obj.hash;
    }
    await btcRechargeToServer(toAddr, newHash, btc2Sat(txRecord.pay).toString(), txRecord.fee, oldHash);
    await sendRawTransactionBTC(signedTX);
    txRecord.toAddr = toAddr;
    txRecord.hash = newHash;
    txRecord.time = new Date().getTime();
    return txRecord;
};


/**
 * 
 * 提现
 */
export const withdraw = async (passwd: string, toAddr: string, currencyName: string, amount: number | string) => {
    if (currencyName === 'BTC') {
        return btcWithdraw(passwd, toAddr, amount);
    } else {
        return ethWithdraw(passwd, toAddr, amount);
    }
};
// eth提现
export const ethWithdraw = async (passwd: string, toAddr: string, amount: number | string) => {
    const close = popNewLoading(getStaticLanguage().transfer.withdraw);
    const secretHash = await VerifyIdentidy(passwd);
    if (!secretHash) {
        close.callback(close.widget);
        popNewMessage(getStaticLanguage().transfer.wrongPsw);

        return;
    }
    const hash = await withdrawFromServer(toAddr, eth2Wei(amount));
    close.callback(close.widget);
    if (hash) {
        popNewMessage(getStaticLanguage().transfer.withdrawSuccess);
        const tx: TxHistory = {
            hash,
            addr: toAddr,
            txType: TxType.Withdraw,
            fromAddr: '',
            toAddr,
            pay: Number(amount),
            time: new Date().getTime(),
            status: TxStatus.Pending,
            confirmedBlockNumber: 0,
            needConfirmedBlockNumber: 0,
            info: '',
            currencyName: 'ETH',
            fee: 0,
            nonce: undefined
        };
        dataCenter.timerUpdateTxWithdraw(tx);
        getWithdrawLogs('ETH');
        updateLocalTx(tx);
    }

    return hash;
};

// btc提现
export const btcWithdraw = async (passwd: string, toAddr: string, amount: number | string) => {
    const close = popNewLoading(getStaticLanguage().transfer.withdraw);
    const secretHash = await VerifyIdentidy(passwd);
    if (!secretHash) {
        close.callback(close.widget);
        popNewMessage(getStaticLanguage().transfer.wrongPsw);

        return;
    }
    const hash = await btcWithdrawFromServer(toAddr, btc2Sat(amount).toString());
    close.callback(close.widget);
    if (hash) {
        popNewMessage(getStaticLanguage().transfer.withdrawSuccess);
        const tx: TxHistory = {
            hash,
            addr: toAddr,
            txType: TxType.Receipt,
            fromAddr: '',
            toAddr,
            pay: Number(amount),
            time: new Date().getTime(),
            status: TxStatus.Pending,
            confirmedBlockNumber: 0,
            needConfirmedBlockNumber: 0,
            info: '',
            currencyName: 'BTC',
            fee: 0,
            nonce: undefined
        };
        dataCenter.timerUpdateTxWithdraw(tx);
        getWithdrawLogs('BTC');
        updateLocalTx(tx);
    }

    return hash;
};